{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# V1"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Punto 1: Clases, Objetos y Pilares de la POO\n",
    "**Objetivo:**\n",
    "Crear una aplicación simple que ilustre los cuatro pilares de la POO: abstracción, encapsulamiento, herencia y polimorfismo. El ejemplo debe estar relacionado con una tienda de libros.\n",
    "\n",
    "**Requerimientos:**\n",
    "- Crea una clase o interfaz `Producto` que represente los atributos básicos de un producto (nombre, precio, agrega uno más).\n",
    "- Crea tres subclases `Libro`, `Revista`, `Video` que hereden de `Producto` y agregue atributos específicos como autor, número de páginas, editorial, volumen, etc. En total cada clase debe tener 2 atributos adicionales.\n",
    "- Implementa los conceptos de encapsulamiento en ambas clases (usa `private` para los atributos y métodos `getters` y `setters`).\n",
    "- Usa el polimorfismo en un método `mostrarInfo()` que se defina en `Producto` y sea sobrescrito en `Libro` para mostrar detalles adicionales.\n",
    "  \n",
    "**Indicaciones adicionales:**\n",
    "- Explica cómo tu código cumple con los cuatro pilares de la POO.\n",
    "- Implementa un método `main` o una clase de prueba en donde crees instancias de `Producto` y `Libro`, y utilices polimorfismo para mostrar la información.\n",
    "\n",
    "**Diagrama UML:**\n",
    "Dibuja el diagrama UML de clases que muestre la relación entre `Producto`, `Libro`, `Revista`. Asegúrate de incluir las clases, atributos, métodos, y las relaciones de herencia y uso de la interfaz.\n",
    "\n",
    "### Solución\n",
    "\n",
    "#### Clases"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {
    "tags": [
     "hide-cell"
    ],
    "vscode": {
     "languageId": "java"
    }
   },
   "outputs": [],
   "source": [
    "abstract class Producto{\n",
    "  private String nombre;\n",
    "  private double precio;\n",
    "  private String tipo;\n",
    "\n",
    "  public Producto(String nombre, double precio, String tipo){\n",
    "    this.nombre = nombre;\n",
    "    this.precio = precio;\n",
    "    this.tipo = tipo;\n",
    "  }\n",
    "\n",
    "  public void setNombre(String nombre){\n",
    "    this.nombre = nombre;\n",
    "  };\n",
    "  public String getNombre(){return nombre;};\n",
    "\n",
    "  public void setPrecio(double precio){\n",
    "    this.precio = precio;\n",
    "  };\n",
    "  public double getPrecio(){ return precio;};\n",
    "\n",
    "  public void setTipo(String tipo){\n",
    "    this.tipo = tipo;\n",
    "  }\n",
    "  public String getTipo(){return tipo;}\n",
    "\n",
    "  public abstract void MostrarInformacion();\n",
    "}\n",
    "\n",
    "class Libro extends Producto{\n",
    "  private String autor;\n",
    "  private int no_paginas;\n",
    "\n",
    "  public Libro(String nombre, double precio, String autor, int no_paginas){\n",
    "    super(nombre, precio, \"Libro\");\n",
    "    this.autor = autor;\n",
    "    this.no_paginas = no_paginas;\n",
    "  }\n",
    "\n",
    "  public void setAutor(String autor){\n",
    "    this.autor = autor;\n",
    "  }\n",
    "  public String getAutor(){\n",
    "    return autor;\n",
    "  }\n",
    "\n",
    "  public void setNoPages(int no_paginas){\n",
    "    this.no_paginas = no_paginas;\n",
    "  }\n",
    "  public int getNoPages(){\n",
    "    return no_paginas;\n",
    "  }\n",
    "\n",
    "  @Override\n",
    "  public void MostrarInformacion(){\n",
    "    System.out.print(\"\"\"\n",
    "      El producto es de tipo %s, el nombre es  %s y cuesta %.1f.\n",
    "      El %s es del autor es %s y el libro tiene %d páginas.\n",
    "      \"\"\".formatted(getTipo(), getNombre(), getPrecio(),\n",
    "                    getTipo(), autor, no_paginas));\n",
    "  };\n",
    "}\n",
    "\n",
    "class Video extends Producto{\n",
    "  private double duracion;\n",
    "  private String formato;\n",
    "\n",
    "  public Video(String nombre, double precio, String formato, double duracion){\n",
    "    super(nombre, precio, \"Video\");\n",
    "    this.duracion = duracion;\n",
    "    this.formato = formato;\n",
    "  }\n",
    "\n",
    "  public void setDuracion(double duracion){\n",
    "    this.duracion = duracion;\n",
    "  }\n",
    "  public double getDuracion(){\n",
    "    return duracion;\n",
    "  }\n",
    "\n",
    "  public void setFormato(String formato){\n",
    "    this.formato = formato;\n",
    "  }\n",
    "  public String getFormato(){\n",
    "    return formato;\n",
    "  }\n",
    "\n",
    "  @Override\n",
    "  public void MostrarInformacion(){\n",
    "    System.out.print(\"\"\"\n",
    "      El producto es de tipo %s, el nombre es  %s y cuesta %.1f.\n",
    "      El %s dura %.1f minutos y esta en formato %s\n",
    "      \"\"\".formatted(getTipo(), getNombre(), getPrecio(),\n",
    "                    getTipo(), duracion, formato));\n",
    "  };\n",
    "}\n",
    "\n",
    "class Revista extends Producto{\n",
    "  private int volumen;\n",
    "  private String editorial;\n",
    "\n",
    "  public Revista(String nombre, double precio, String editorial, int volumen){\n",
    "    super(nombre, precio, \"Revista\");\n",
    "    this.editorial = editorial;\n",
    "    this.volumen = volumen;\n",
    "  }\n",
    "\n",
    "  public void setVolumen(int volumen){\n",
    "    this.volumen = volumen;\n",
    "  }\n",
    "  public int getVolumen(){\n",
    "    return volumen;\n",
    "  }\n",
    "\n",
    "  public void setEditorial(String editorial){\n",
    "    this.editorial = editorial;\n",
    "  }\n",
    "  public String getEditorial(){\n",
    "    return editorial;\n",
    "  }\n",
    "\n",
    "  @Override\n",
    "  public void MostrarInformacion(){\n",
    "    System.out.print(\"\"\"\n",
    "      El producto es de tipo %s, el nombre es  %s y cuesta %.1f.\n",
    "      La %s es de la editorial %s y es el volumen %d.\n",
    "      \"\"\".formatted(getTipo(), getNombre(), getPrecio(),\n",
    "           getTipo(), editorial, volumen));\n",
    "  };\n",
    "}"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Clase de Prueba"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {
    "tags": [
     "hide-cell"
    ],
    "vscode": {
     "languageId": "java"
    }
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "El producto es de tipo Libro, el nombre es  Harry Potter y cuesta 150000,0.\n",
      "El Libro es del autor es J. K. Rowling y el libro tiene 223 páginas.\n",
      "El producto es de tipo Video, el nombre es  Harry Potter y cuesta 20000,0.\n",
      "El Video dura 152,0 minutos y esta en formato MP4\n",
      "El producto es de tipo Revista, el nombre es  Vogue y cuesta 30000,0.\n",
      "La Revista es de la editorial Vogue y es el volumen 1.\n"
     ]
    }
   ],
   "source": [
    "class TestPunto1{\n",
    "  public static void main(String[] args){\n",
    "    Libro HarryPoter = new Libro(\"Harry Potter\", 150000, \"J. K. Rowling\", 223);\n",
    "    HarryPoter.MostrarInformacion();\n",
    "\n",
    "    Video HarryPoterMovie = new Video(\"Harry Potter\", 20000, \"MP4\", 152);\n",
    "    HarryPoterMovie.MostrarInformacion();\n",
    "\n",
    "    Revista Vogue = new Revista(\"Vogue\", 30000, \"Vogue\", 1);\n",
    "    Vogue.MostrarInformacion();\n",
    "  }\n",
    "}\n",
    "new TestPunto1().main(null);"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Diagrama UML de clases\n",
    "\n",
    "```{toggle} Diagrama UML de clases\n",
    "<figure>\n",
    "  <img src=\"../../../images/exam1_Punto1.png\" alt=\"\" style=\"width:95%\">\n",
    "  <figcaption> Diagrama UML</figcaption>\n",
    "</figure> \n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Punto 2: Diagramas UML y Aplicación Completa\n",
    "\n",
    "**Objetivo:**\n",
    "Desarrollar una solución completa para un sistema que gestione una flota de vehículos utilizando herencia e interfaces. Luego, realizar el diagrama UML de clases correspondiente.\n",
    "\n",
    "**Requerimientos:**\n",
    "- Crea una clase `Vehiculo` con los atributos `marca`, `modelo`, y `año`.\n",
    "- Crea dos subclases: `Auto` y `Moto`. `Auto` tiene atributos adicionales como `numeroDePuertas`, y `Moto` tiene `cilindrada`.\n",
    "- Crea una interfaz `Conducible` que tenga el método `conducir()`. Tanto `Auto` como `Moto` deben implementar esta interfaz.\n",
    "- En el método `main`, crea instancias de `Auto` y `Moto` y utiliza polimorfismo para llamar al método `conducir()`.\n",
    "\n",
    "**Diagrama UML:**\n",
    "Dibuja el diagrama UML de clases que muestre la relación entre `Vehiculo`, `Auto`, `Moto`, y la interfaz `Conducible`. Asegúrate de incluir las clases, atributos, métodos, y las relaciones de herencia y uso de la interfaz.\n",
    "\n",
    "### Solución\n",
    "\n",
    "#### Clases e Interfaces"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {
    "tags": [
     "hide-cell"
    ],
    "vscode": {
     "languageId": "java"
    }
   },
   "outputs": [],
   "source": [
    "class Vehiculo {\n",
    "  public String marca;\n",
    "  public String modelo;\n",
    "  public int año;\n",
    "\n",
    "  public static void main (String[] args){\n",
    "    Auto chevrolet = new Auto();\n",
    "    chevrolet.marca = \"Chevrolet\";\n",
    "    chevrolet.modelo = \"CRUZE\";\n",
    "    chevrolet.año = 2015;\n",
    "    chevrolet.numeroDePuertas = 2;\n",
    "    chevrolet.conducir();\n",
    "    \n",
    "    \n",
    "    Moto pulsar200 = new Moto();\n",
    "    pulsar200.marca = \"Pulsar\";\n",
    "    pulsar200.modelo = \"NS\";\n",
    "    pulsar200.año = 2020;\n",
    "    pulsar200.cilindrada = 200;\n",
    "    pulsar200.conducir();\n",
    "  }\n",
    "}\n",
    "\n",
    "interface Conducible{\n",
    "  public void conducir();\n",
    "}\n",
    "\n",
    "class Auto extends Vehiculo implements Conducible{\n",
    "  public int numeroDePuertas;\n",
    "\n",
    "  @Override\n",
    "  public void conducir(){System.out.println(\"Conduciendo carro...\");};\n",
    "}\n",
    "\n",
    "class Moto extends Vehiculo implements Conducible{\n",
    "  public int cilindrada;\n",
    "\n",
    "  @Override\n",
    "  public void conducir(){System.out.println(\"Conduciendo moto...\");};\n",
    "}"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "vscode": {
     "languageId": "java"
    }
   },
   "source": [
    "#### Prueba de Clase"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {
    "tags": [
     "hide-cell"
    ],
    "vscode": {
     "languageId": "java"
    }
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Conduciendo carro...\n",
      "\n",
      "Conduciendo moto...\n"
     ]
    }
   ],
   "source": [
    "new Vehiculo().main(null);"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Diagrama UML de Clases\n",
    "```{toggle} Diagrama UML de clases\n",
    ":show:\n",
    "<figure>\n",
    "  <img src=\"../../../images/exam1_Punto2.png\" alt=\"\" style=\"width:80%\">\n",
    "  <figcaption>Diagrama de clases UML.</figcaption>\n",
    "</figure> \n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Punto 3: De Diagrama UML a Código\n",
    "\n",
    "Implementen el código del siguiente diagrama UML de clases\n",
    "\n",
    "<figure>\n",
    "  <img src=\"https://media.geeksforgeeks.org/wp-content/uploads/20240308165524/Class-Diagram-example.webp\" alt=\"\" style=\"width:90%\">\n",
    "  <figcaption> </figcaption>\n",
    "</figure> \n",
    "\n",
    "### Solución\n",
    "\n",
    "#### Clases"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {
    "tags": [
     "hide-cell"
    ],
    "vscode": {
     "languageId": "java"
    }
   },
   "outputs": [],
   "source": [
    "class Animal{\n",
    "  public int age;\n",
    "  public String gender;\n",
    "\n",
    "  public boolean isMammal(){ return false;};\n",
    "  public String mate(){ return \"single\";};\n",
    "}\n",
    "\n",
    "class Duck extends Animal{\n",
    "  public String beakColr = \"Yellow\";\n",
    "\n",
    "  public void Swim(){System.out.println(\"Swimming\");};\n",
    "  public void quack(){System.out.println(\"quack\");};\n",
    "} \n",
    "\n",
    "class Fish extends Animal{\n",
    "  private int sizeInFt;\n",
    "  private boolean canEat;\n",
    "  \n",
    "  private void Swim(){System.out.println(\"Swimming\");};\n",
    "}\n",
    "\n",
    "\n",
    "class Zebra extends Animal{\n",
    "  public boolean is_wild;\n",
    "  \n",
    "  public void run(){System.out.println(\"running\");};\n",
    "  \n",
    "  @Override\n",
    "  public boolean isMammal(){ return true;};\n",
    "\n",
    "  @Override\n",
    "  public String mate(){ return \"divorced\";};\n",
    "}"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Pruebla de Clase"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {
    "tags": [
     "hide-cell"
    ],
    "vscode": {
     "languageId": "java"
    }
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "quack\n",
      "Is mammal? false\n",
      "Has a mate? single\n",
      "Beak color: Yellow\n",
      "Is mammal? false\n",
      "Has a mate? single\n",
      "Is mammal? true\n",
      "Has a mate? divorced\n",
      "Is wild? true\n"
     ]
    }
   ],
   "source": [
    "public class TestAnimals{\n",
    "  public static void main(String[] args){\n",
    "    Duck pato = new Duck();\n",
    "    pato.quack();\n",
    "    System.out.print(\"\"\"\n",
    "        Is mammal? %b\n",
    "        Has a mate? %s\n",
    "        Beak color: %s\n",
    "        \"\"\".formatted(pato.isMammal(),pato.mate(), pato.beakColr));\n",
    "\n",
    "    Fish pescado = new Fish();\n",
    "    //pescado.Swim(); // no funciona porque es privado\n",
    "    System.out.print(\"\"\"\n",
    "        Is mammal? %b\n",
    "        Has a mate? %s\n",
    "          \"\"\".formatted(pescado.isMammal(), pescado.mate()));\n",
    "    //System.out.println(\"Can eat? \"+pescado.canEat); // No funciona porque es privado, se necesitaria un get\n",
    "\n",
    "    Zebra zebra = new Zebra();\n",
    "    zebra.is_wild = true; // como es publico puedo darle un valor sin usar setter\n",
    "    System.out.print(\"\"\"\n",
    "        Is mammal? %b\n",
    "        Has a mate? %s\n",
    "        Is wild? %b\n",
    "          \"\"\".formatted(zebra.isMammal(), zebra.mate(), zebra.is_wild));\n",
    "  }\n",
    "}\n",
    "\n",
    "new TestAnimals().main(null);"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Java",
   "language": "java",
   "name": "java"
  },
  "language_info": {
   "codemirror_mode": "java",
   "file_extension": ".jshell",
   "mimetype": "text/x-java-source",
   "name": "Java",
   "pygments_lexer": "java",
   "version": "23.0.1+11-39"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
